@@ -1,10 +1,14 @@
 /*
-   Copyright (C) 2002 MySQL AB
+      Copyright (C) 2002-2004 MySQL AB
 
       This program is free software; you can redistribute it and/or modify
-      it under the terms of the GNU General Public License as published by
-      the Free Software Foundation; either version 2 of the License, or
-      (at your option) any later version.
+      it under the terms of version 2 of the GNU General Public License as 
+      published by the Free Software Foundation.
+
+      There are special exceptions to the terms and conditions of the GPL 
+      as it is applied to this software. View the full text of the 
+      exception in file EXCEPTIONS-CONNECTOR-J in the directory of this 
+      software distribution.
 
       This program is distributed in the hope that it will be useful,
       but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,10 +19,14 @@
       along with this program; if not, write to the Free Software
       Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+
+
  */
 package com.mysql.jdbc;
 
 import java.io.UnsupportedEncodingException;
+import java.sql.SQLException;
+import java.sql.Types;
 
 
 /**
@@ -26,7 +34,7 @@ import java.io.UnsupportedEncodingException;
  * ResultSet
  *
  * @author Mark Matthews
- * @version $Id: Field.java,v 1.15.2.4 2003/03/21 13:43:50 mmatthew Exp $
+ * @version $Id: Field.java,v 1.15.4.24 2005/02/17 21:42:27 mmatthews Exp $
  */
 public class Field {
     //~ Static fields/initializers ---------------------------------------------
@@ -39,11 +47,8 @@ public class Field {
     private Connection connection = null;
     private String charsetName = null;
     private String databaseName = null;
-    private String defaultValue = null;
     private String fullName = null;
-    private String fullNameWithDatabase = null;
     private String fullOriginalName = null;
-    private String fullOriginalNameWithDatabase = null;
     private String name; // The Field name
     private String originalColumnName = null;
     private String originalTableName = null;
@@ -76,6 +81,8 @@ public class Field {
     private int tableNameLength;
     private int tableNameStart;
     private short colFlag;
+    
+    private String collationName = null;
 
     //~ Constructors -----------------------------------------------------------
 
@@ -86,9 +93,9 @@ public class Field {
         this.tableName = tableName;
         this.name = columnName;
         this.length = length;
-        sqlType = jdbcType;
-        colFlag = 0;
-        colDecimals = 0;
+        this.sqlType = jdbcType;
+        this.colFlag = 0;
+        this.colDecimals = 0;
     }
 
     /**
@@ -96,7 +103,7 @@ public class Field {
      */
     Field(Connection conn, byte[] buffer, int nameStart, int nameLength,
         int tableNameStart, int tableNameLength, int length, int mysqlType,
-        short colFlag, int colDecimals) {
+        short colFlag, int colDecimals) throws SQLException {
         this(conn, buffer, -1, -1, tableNameStart, tableNameLength, -1, -1,
             nameStart, nameLength, -1, -1, length, mysqlType, colFlag,
             colDecimals, -1, -1, NO_CHARSET_INFO);
@@ -112,7 +119,7 @@ public class Field {
         int nameLength, int originalColumnNameStart,
         int originalColumnNameLength, int length, int mysqlType, short colFlag,
         int colDecimals, int defaultValueStart, int defaultValueLength,
-        int charsetIndex) {
+        int charsetIndex) throws SQLException {
         this.connection = conn;
         this.buffer = buffer;
         this.nameStart = nameStart;
@@ -137,32 +144,47 @@ public class Field {
         this.defaultValueStart = defaultValueStart;
         this.defaultValueLength = defaultValueLength;
 
-        // Map MySqlTypes to java.sql Types
-        sqlType = MysqlDefs.mysqlToJavaType(mysqlType);
+        // Re-map to 'real' blob type, if we're a BLOB
 
-        // If we're not running 4.1 or newer, use the connection's
-        // charset
-        if (charsetIndex != NO_CHARSET_INFO) {
-            this.charsetIndex = charsetIndex;
-			this.charsetName = CharsetMapping.INDEX_TO_CHARSET[this.charsetIndex];
-            
-            // Punt
-            if (this.charsetName == null) {
-				this.charsetName = this.connection.getEncoding();
-            }  
-        } else {
-            this.charsetName = this.connection.getEncoding();
+        if (this.mysqlType == MysqlDefs.FIELD_TYPE_BLOB) {
+        	setBlobTypeBasedOnLength();
+        }
+        
+        // Map MySqlTypes to java.sql Types
+        this.sqlType = MysqlDefs.mysqlToJavaType(this.mysqlType);
+        
+        if (this.sqlType == Types.TINYINT && this.length == 1 
+        		&& this.connection.getTinyInt1isBit()) {
+        	this.sqlType = Types.BIT;
+        }
+        
+        if (this.mysqlType == MysqlDefs.FIELD_TYPE_BIT) {
+        	if (this.length == 0) {
+        		this.sqlType = Types.BIT;
+        	} else {
+        		this.sqlType = Types.VARBINARY;
+        		this.colFlag |= 128; // we need to pretend this is a full
+        		this.colFlag |= 16; // binary blob
+        	}
         }
 
+	
+        // If we're not running 4.1 or newer, use the connection's
+        // charset
+        this.charsetIndex = charsetIndex;
+        
+        this.charsetName = this.connection.getCharsetNameForIndex(this.charsetIndex);
+
+		
         boolean isBinary = isBinary();
 
         //
         // Handle TEXT type (special case), Fix proposed by Peter McKeown
         //
-        if ((sqlType == java.sql.Types.LONGVARBINARY) && !isBinary) {
-            sqlType = java.sql.Types.LONGVARCHAR;
-        } else if ((sqlType == java.sql.Types.VARBINARY) && !isBinary) {
-            sqlType = java.sql.Types.VARCHAR;
+        if ((this.sqlType == java.sql.Types.LONGVARBINARY) && !isBinary) {
+            this.sqlType = java.sql.Types.LONGVARCHAR;
+        } else if ((this.sqlType == java.sql.Types.VARBINARY) && !isBinary) {
+            this.sqlType = java.sql.Types.VARCHAR;
         }
 
         //
@@ -200,7 +222,7 @@ public class Field {
      * @return DOCUMENT ME!
      */
     public boolean isAutoIncrement() {
-        return ((colFlag & AUTO_INCREMENT_FLAG) > 0);
+        return ((this.colFlag & AUTO_INCREMENT_FLAG) > 0);
     }
 
     /**
@@ -209,7 +231,7 @@ public class Field {
      * @return DOCUMENT ME!
      */
     public boolean isBinary() {
-        return ((colFlag & 128) > 0);
+        return ((this.colFlag & 128) > 0);
     }
 
     /**
@@ -218,7 +240,7 @@ public class Field {
      * @return DOCUMENT ME!
      */
     public boolean isBlob() {
-        return ((colFlag & 16) > 0);
+        return ((this.colFlag & 16) > 0);
     }
 
     /**
@@ -230,6 +252,76 @@ public class Field {
     public String getCharacterSet() {
         return this.charsetName;
     }
+    
+    public int getMaxBytesPerCharacter() throws SQLException {
+    	return this.connection.getMaxBytesPerChar((String)CharsetMapping.JAVA_TO_MYSQL_CHARSET_MAP.get(getCharacterSet()));
+    }
+    
+    public synchronized String getCollation() throws SQLException {
+    	if (this.collationName == null) {
+    		if (this.connection != null) {
+    			if (this.connection.versionMeetsMinimum(4, 1, 0)) {
+    				java.sql.DatabaseMetaData dbmd = this.connection.getMetaData();
+    			
+    				String quotedIdStr = dbmd.getIdentifierQuoteString();
+    			
+    				if (" ".equals(quotedIdStr)) { //$NON-NLS-1$
+    					quotedIdStr = ""; //$NON-NLS-1$
+    				}
+    				
+    				String csCatalogName = getDatabaseName();
+    				String csTableName = getOriginalTableName();
+    				String csColumnName = getOriginalName();
+    				
+    				if (csCatalogName != null && csCatalogName.length() != 0
+    						&& csTableName != null && csTableName.length() != 0
+    						&& csColumnName != null && csColumnName.length() != 0) {
+    					StringBuffer queryBuf = new StringBuffer(csCatalogName.length() + csTableName.length() + 28);
+    					queryBuf.append("SHOW FULL COLUMNS FROM "); //$NON-NLS-1$
+    					queryBuf.append(quotedIdStr);
+    					queryBuf.append(csCatalogName);
+    					queryBuf.append(quotedIdStr);
+    					queryBuf.append("."); //$NON-NLS-1$
+    					queryBuf.append(quotedIdStr);
+    					queryBuf.append(csTableName);
+						queryBuf.append(quotedIdStr);
+
+						java.sql.Statement collationStmt = null;
+						java.sql.ResultSet collationRs = null;
+
+						try {
+							collationStmt = this.connection.createStatement();
+							
+							collationRs = collationStmt.executeQuery(queryBuf.toString());
+
+							while (collationRs.next()) {
+								if (csColumnName.equals(collationRs.getString("Field"))) { //$NON-NLS-1$
+									this.collationName = collationRs.getString("Collation"); //$NON-NLS-1$
+
+									break;
+								}
+							}
+						} finally {
+							if (collationRs != null) {
+								collationRs.close();
+								collationRs = null;
+							}
+
+							if (collationStmt != null) {
+								collationStmt.close();
+								collationStmt = null;
+							}
+						}
+
+    				}
+    			}
+    			
+    		}
+    		
+    	}
+    	
+    	return this.collationName;
+    }
 
     /**
      * DOCUMENT ME!
@@ -263,19 +355,19 @@ public class Field {
      * @return DOCUMENT ME!
      */
     public String getFullName() {
-        if (fullName == null) {
+        if (this.fullName == null) {
             StringBuffer fullNameBuf = new StringBuffer(getTableName().length()
                     + 1 + getName().length());
-            fullNameBuf.append(tableName);
+            fullNameBuf.append(this.tableName);
 
             // much faster to append a char than a String
             fullNameBuf.append('.');
-            fullNameBuf.append(name);
-            fullName = fullNameBuf.toString();
+            fullNameBuf.append(this.name);
+            this.fullName = fullNameBuf.toString();
             fullNameBuf = null;
         }
 
-        return fullName;
+        return this.fullName;
     }
 
     /**
@@ -290,7 +382,7 @@ public class Field {
             return null; // we don't have this information
         }
 
-        if (fullName == null) {
+        if (this.fullName == null) {
             StringBuffer fullOriginalNameBuf = new StringBuffer(getOriginalTableName()
                                                                     .length()
                     + 1 + getOriginalName().length());
@@ -312,7 +404,7 @@ public class Field {
      * @return DOCUMENT ME!
      */
     public int getLength() {
-        return length;
+        return this.length;
     }
 
     /**
@@ -321,7 +413,7 @@ public class Field {
      * @return DOCUMENT ME!
      */
     public boolean isMultipleKey() {
-        return ((colFlag & 8) > 0);
+        return ((this.colFlag & 8) > 0);
     }
 
     /**
@@ -330,7 +422,7 @@ public class Field {
      * @return DOCUMENT ME!
      */
     public int getMysqlType() {
-        return mysqlType;
+        return this.mysqlType;
     }
 
     /**
@@ -343,7 +435,7 @@ public class Field {
             this.name = getStringFromBytes(this.nameStart, this.nameLength);
         }
 
-        return name;
+        return this.name;
     }
 
     /**
@@ -397,7 +489,7 @@ public class Field {
      * @return DOCUMENT ME!
      */
     public boolean isPrimaryKey() {
-        return ((colFlag & 2) > 0);
+        return ((this.colFlag & 2) > 0);
     }
 
     /**
@@ -406,7 +498,7 @@ public class Field {
      * @return DOCUMENT ME!
      */
     public int getSQLType() {
-        return sqlType;
+        return this.sqlType;
     }
 
     /**
@@ -424,11 +516,11 @@ public class Field {
      * @return DOCUMENT ME!
      */
     public String getTableName() {
-        if (tableName == null) {
-            tableName = getStringFromBytes(tableNameStart, tableNameLength);
+        if (this.tableName == null) {
+            this.tableName = getStringFromBytes(this.tableNameStart, this.tableNameLength);
         }
 
-        return tableName;
+        return this.tableName;
     }
 
     /**
@@ -437,7 +529,7 @@ public class Field {
      * @return DOCUMENT ME!
      */
     public boolean isUniqueKey() {
-        return ((colFlag & 4) > 0);
+        return ((this.colFlag & 4) > 0);
     }
 
     /**
@@ -446,7 +538,7 @@ public class Field {
      * @return DOCUMENT ME!
      */
     public boolean isUnsigned() {
-        return ((colFlag & 32) > 0);
+        return ((this.colFlag & 32) > 0);
     }
 
     /**
@@ -455,7 +547,7 @@ public class Field {
      * @return DOCUMENT ME!
      */
     public boolean isZeroFill() {
-        return ((colFlag & 64) > 0);
+        return ((this.colFlag & 64) > 0);
     }
 
     /**
@@ -464,15 +556,49 @@ public class Field {
      * @return DOCUMENT ME!
      */
     public String toString() {
-        return getFullName();
+    	return this.getDatabaseName() + " . " +  this.getTableName() + "(" + this.getOriginalTableName() + ") . " + this.getName() + "(" + this.getOriginalName() + ")" + ", Mysql type: " + getMysqlType(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ //$NON-NLS-6$
+       
     }
 
     int getDecimals() {
-        return colDecimals;
+        return this.colDecimals;
     }
 
     boolean isNotNull() {
-        return ((colFlag & 1) > 0);
+        return ((this.colFlag & 1) > 0);
+    }
+    
+    boolean isOpaqueBinary() throws SQLException {
+    	
+    	if (this.charsetIndex == 63 && isBinary()) {
+    		return true;
+    	}
+    	
+    	return (this.connection.versionMeetsMinimum(4, 1, 0) &&
+    			"binary".equalsIgnoreCase(getCharacterSet()));
+    			
+    }
+    /**
+     * Is this field _definitely_ not writable?
+     * 
+     * @return true if this field can not be written to in an INSERT/UPDATE
+     * statement.
+     */
+    boolean isReadOnly() throws SQLException {
+    	if (this.connection.versionMeetsMinimum(4, 1, 0)) {
+    		String orgColumnName = getOriginalName();
+    		String orgTableName = getOriginalTableName();
+    		
+    		return !(orgColumnName != null && orgColumnName.length() > 0 &&
+    				orgTableName != null && orgTableName.length() > 0);
+    	}
+    		
+    	return false; // we can't tell definitively in this case.
+    }
+    
+    void setMysqlType(int type) {
+    	this.mysqlType = type;
+    	this.sqlType = MysqlDefs.mysqlToJavaType(this.mysqlType);
     }
 
     /**
@@ -486,21 +612,23 @@ public class Field {
 
         String stringVal = null;
 
-        if (connection != null) {
-            if (connection.useUnicode()) {
-                String encoding = connection.getEncoding();
+        if (this.connection != null) {
+            if (this.connection.getUseUnicode()) {
+            	String encoding = this.connection.getCharacterSetMetadata();
+                
+                if (encoding == null) {
+                	encoding = connection.getEncoding();
+                }
 
                 if (encoding != null) {
                     SingleByteCharsetConverter converter = null;
 
-                    try {
-                        converter = SingleByteCharsetConverter.getInstance(encoding);
-                    } catch (UnsupportedEncodingException uee) {
-                        // ignore, code further down handles this
+                    if (this.connection != null) {
+                        converter = this.connection.getCharsetConverter(encoding);
                     }
 
                     if (converter != null) { // we have a converter
-                        stringVal = converter.toString(buffer, stringStart,
+                        stringVal = converter.toString(this.buffer, stringStart,
                                 stringLength);
                     } else {
                         // we have no converter, use JVM converter 
@@ -510,33 +638,50 @@ public class Field {
                         int pos = 0;
 
                         for (int i = stringStart; i < endIndex; i++) {
-                            stringBytes[pos++] = buffer[i];
+                            stringBytes[pos++] = this.buffer[i];
                         }
 
                         try {
                             stringVal = new String(stringBytes, encoding);
                         } catch (UnsupportedEncodingException ue) {
                             throw new RuntimeException(
-                                "Unsupported character encoding '" + encoding
-                                + "'");
+                                Messages.getString("Field.12") + encoding //$NON-NLS-1$
+                                + Messages.getString("Field.13")); //$NON-NLS-1$
                         }
                     }
                 } else {
                     // we have no encoding, use JVM standard charset
-                    stringVal = StringUtils.toAsciiString(buffer, stringStart,
+                    stringVal = StringUtils.toAsciiString(this.buffer, stringStart,
                             stringLength);
                 }
             } else {
                 // we are not using unicode, so use JVM standard charset 
-                stringVal = StringUtils.toAsciiString(buffer, stringStart,
+                stringVal = StringUtils.toAsciiString(this.buffer, stringStart,
                         stringLength);
             }
         } else {
             // we don't have a connection, so punt 
-            stringVal = StringUtils.toAsciiString(buffer, stringStart,
+            stringVal = StringUtils.toAsciiString(this.buffer, stringStart,
                     stringLength);
         }
 
         return stringVal;
     }
+
+	//
+	// MySQL only has one protocol-level BLOB type that it exposes
+	// which is FIELD_TYPE_BLOB, although we can divine what the
+	// actual type is by the length reported ...
+	//
+	private void setBlobTypeBasedOnLength() {
+		 if (this.length == MysqlDefs.LENGTH_TINYBLOB) {
+		 	this.mysqlType = MysqlDefs.FIELD_TYPE_TINY_BLOB;
+		 } else if (this.length == MysqlDefs.LENGTH_BLOB) {
+		 	this.mysqlType = MysqlDefs.FIELD_TYPE_BLOB;
+		 } else if (this.length == MysqlDefs.LENGTH_MEDIUMBLOB) {
+		 	this.mysqlType = MysqlDefs.FIELD_TYPE_MEDIUM_BLOB;
+		 } else if (this.length == MysqlDefs.LENGTH_LONGBLOB) {
+		 	this.mysqlType = MysqlDefs.FIELD_TYPE_LONG_BLOB;
+		 }
+	}
 }
