@@ -1,10 +1,14 @@
 /*
-   Copyright (C) 2002 MySQL AB
+      Copyright (C) 2002-2004 MySQL AB
 
       This program is free software; you can redistribute it and/or modify
-      it under the terms of the GNU General Public License as published by
-      the Free Software Foundation; either version 2 of the License, or
-      (at your option) any later version.
+      it under the terms of version 2 of the GNU General Public License as 
+      published by the Free Software Foundation.
+
+      There are special exceptions to the terms and conditions of the GPL 
+      as it is applied to this software. View the full text of the 
+      exception in file EXCEPTIONS-CONNECTOR-J in the directory of this 
+      software distribution.
 
       This program is distributed in the hope that it will be useful,
       but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,6 +19,8 @@
       along with this program; if not, write to the Free Software
       Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+
+
  */
 package com.mysql.jdbc;
 
@@ -25,10 +31,12 @@ package com.mysql.jdbc;
  * @author Mark Matthews
  */
 public class NotImplemented extends java.sql.SQLException {
+    //~ Constructors -----------------------------------------------------------
+
     /**
      * Creates a new NotImplemented object.
      */
     public NotImplemented() {
-        super("Feature not implemented", "S1C00");
+        super(Messages.getString("NotImplemented.0"), SQLError.SQL_STATE_DRIVER_NOT_CAPABLE); //$NON-NLS-1$
     }
 }
