@@ -1,10 +1,14 @@
 /*
-   Copyright (C) 2002 MySQL AB
+      Copyright (C) 2002-2004 MySQL AB
 
       This program is free software; you can redistribute it and/or modify
-      it under the terms of the GNU General Public License as published by
-      the Free Software Foundation; either version 2 of the License, or
-      (at your option) any later version.
+      it under the terms of version 2 of the GNU General Public License as 
+      published by the Free Software Foundation.
+
+      There are special exceptions to the terms and conditions of the GPL 
+      as it is applied to this software. View the full text of the 
+      exception in file EXCEPTIONS-CONNECTOR-J in the directory of this 
+      software distribution.
 
       This program is distributed in the hope that it will be useful,
       but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,6 +19,8 @@
       along with this program; if not, write to the Free Software
       Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+
+
  */
 package com.mysql.jdbc;
 
@@ -26,50 +32,53 @@ import java.sql.Types;
  * A ResultSetMetaData object can be used to find out about the types and
  * properties of the columns in a ResultSet
  *
- * @see java.sql.ResultSetMetaData
  * @author Mark Matthews
- * @version $Id: ResultSetMetaData.java,v 1.12.2.3 2003/03/10 13:02:48 mmatthew Exp $
+ * @version $Id: ResultSetMetaData.java,v 1.12.4.21 2004/12/09 22:02:19 mmatthew Exp $
+ *
+ * @see java.sql.ResultSetMetaData
  */
 public class ResultSetMetaData implements java.sql.ResultSetMetaData {
     Field[] fields;
 
     /**
-            * Initialise for a result with a tuple set and
-     * a field descriptor set
+     * Initialise for a result with a tuple set and a field descriptor set
      *
      * @param fields the array of field descriptors
-
      */
     public ResultSetMetaData(Field[] fields) {
         this.fields = fields;
     }
 
     /**
-     * Is the column automatically numbered (and thus read-only)
-     *
-     * MySQL Auto-increment columns are not read only,
-     * so to conform to the spec, this method returns false.
+     * Is the column automatically numbered (and thus read-only) MySQL
+     * Auto-increment columns are not read only, so to conform to the spec,
+     * this method returns false.
      *
      * @param column the first column is 1, the second is 2...
+     *
      * @return true if so
-     * @throws java.sql.SQLException if a database access error occurs
+     *
+     * @throws SQLException if a database access error occurs
      */
-    public boolean isAutoIncrement(int column) throws java.sql.SQLException {
+    public boolean isAutoIncrement(int column) throws SQLException {
         Field f = getField(column);
 
         return f.isAutoIncrement();
     }
 
     /**
-     * Does a column's case matter? ASSUMPTION: Any field that is
-     * not obviously case insensitive is assumed to be case sensitive
+     * Does a column's case matter?
      *
      * @param column the first column is 1, the second is 2...
+     *
      * @return true if so
+     *
      * @throws java.sql.SQLException if a database access error occurs
      */
     public boolean isCaseSensitive(int column) throws java.sql.SQLException {
-        int sqlType = getField(column).getSQLType();
+        Field field = getField(column);
+
+        int sqlType = field.getSQLType();
 
         switch (sqlType) {
         case Types.BIT:
@@ -85,6 +94,18 @@ public class ResultSetMetaData implements java.sql.ResultSetMetaData {
         case Types.TIMESTAMP:
             return false;
 
+        case Types.CHAR:
+        case Types.VARCHAR:
+        case Types.LONGVARCHAR:
+
+            if (field.isBinary()) {
+                return true;
+            }
+
+            String collationName = field.getCollation();
+
+            return ((collationName != null) && !collationName.endsWith("_ci"));
+
         default:
             return true;
         }
@@ -94,15 +115,54 @@ public class ResultSetMetaData implements java.sql.ResultSetMetaData {
      * What's a column's table's catalog name?
      *
      * @param column the first column is 1, the second is 2...
+     *
      * @return catalog name, or "" if not applicable
-     * @throws java.sql.SQLException if a database access error occurs
+     *
+     * @throws SQLException if a database access error occurs
      */
-    public String getCatalogName(int column) throws java.sql.SQLException {
+    public String getCatalogName(int column) throws SQLException {
         Field f = getField(column);
 
         String database = f.getDatabaseName();
 
-        return (database == null) ? "" : database;
+        return (database == null) ? "" : database; //$NON-NLS-1$
+    }
+
+    /**
+     * What's the Java character encoding name for the given column?
+     *
+     * @param column the first column is 1, the second is 2, etc.
+     *
+     * @return the Java character encoding name for the given column, or null
+     *         if no Java character encoding maps to the MySQL character set
+     *         for the given column.
+     *
+     * @throws SQLException if an invalid column index is given.
+     */
+    public String getColumnCharacterEncoding(int column)
+        throws SQLException {
+        String mysqlName = getColumnCharacterSet(column);
+
+        String javaName = null;
+
+        if (mysqlName != null) {
+            javaName = (String) CharsetMapping.MYSQL_TO_JAVA_CHARSET_MAP.get(mysqlName);
+        }
+
+        return javaName;
+    }
+
+    /**
+     * What's the MySQL character set name for the given column?
+     *
+     * @param column the first column is 1, the second is 2, etc.
+     *
+     * @return the MySQL character set name for the given column
+     *
+     * @throws SQLException if an invalid column index is given.
+     */
+    public String getColumnCharacterSet(int column) throws SQLException {
+        return getField(column).getCharacterSet();
     }
 
     //--------------------------JDBC 2.0-----------------------------------
@@ -110,91 +170,132 @@ public class ResultSetMetaData implements java.sql.ResultSetMetaData {
     /**
      * JDBC 2.0
      *
-     * <p>Return the fully qualified name of the Java class whose instances
-     * are manufactured if ResultSet.getObject() is called to retrieve a value
+     * <p>
+     * Return the fully qualified name of the Java class whose instances are
+     * manufactured if ResultSet.getObject() is called to retrieve a value
      * from the column.  ResultSet.getObject() may return a subClass of the
      * class returned by this method.
+     * </p>
      *
      * @param column the column number to retrieve information for
-     * @return the fully qualified name of the Java class whose instances
-     * are manufactured if ResultSet.getObject() is called to retrieve a value
-     * from the column.
+     *
+     * @return the fully qualified name of the Java class whose instances are
+     *         manufactured if ResultSet.getObject() is called to retrieve a
+     *         value from the column.
      *
      * @throws SQLException if an error occurs
      */
     public String getColumnClassName(int column) throws SQLException {
         Field f = getField(column);
 
+        // From JDBC-3.0 spec, Table B-3
+        //
+        //  JDBC Type Java Object Type
+        //
+        // CHAR 			String
+        // VARCHAR 			String
+        // LONGVARCHAR 		String
+        // NUMERIC 			java.math.BigDecimal
+        // DECIMAL 			java.math.BigDecimal
+        // BIT 				Boolean
+        // BOOLEAN 			Boolean
+        // TINYINT 			Integer
+        // SMALLINT 		Integer
+        // INTEGER 			Integer
+        // BIGINT 			Long
+        // REAL 			Float
+        // FLOAT 			Double
+        // DOUBLE 			Double
+        // BINARY 			byte[]
+        // VARBINARY 		byte[]
+        // LONGVARBINARY 	byte[]
+        // DATE 			java.sql.Date
+        // TIME 			java.sql.Time
+        // TIMESTAMP 		java.sql.Timestamp
+        // DISTINCT 		Object type of underlying type
+        // CLOB 			Clob
+        // BLOB 			Blob
+        // ARRAY 			Array
+        // STRUCT 			Struct or SQLData
+        // REF 				Ref
+        // DATALINK 		java.net.URL
+        // JAVA_OBJECT 		underlying Java class
         switch (f.getSQLType()) {
         case Types.BIT:
-            return "java.lang.Boolean";
+        case Types.BOOLEAN:
+            return "java.lang.Boolean"; //$NON-NLS-1$
 
         case Types.TINYINT:
 
-            if (f.isUnsigned()) {
-                return "java.lang.Integer";
-            } else {
-                return "java.lang.Byte";
+            if (!f.isUnsigned()) {
+                return "java.lang.Integer"; //$NON-NLS-1$
             }
 
+            return "java.lang.Integer"; //$NON-NLS-1$
+
         case Types.SMALLINT:
 
-            if (f.isUnsigned()) {
-                return "java.lang.Integer";
-            } else {
-                return "java.lang.Short";
+            if (!f.isUnsigned()) {
+                return "java.lang.Integer"; //$NON-NLS-1$
             }
 
+            return "java.lang.Integer"; //$NON-NLS-1$
+
         case Types.INTEGER:
 
-            if (f.isUnsigned()) {
-                return "java.lang.Long";
-            } else {
-                return "java.lang.Integer";
+            if (!f.isUnsigned()) {
+                return "java.lang.Integer"; //$NON-NLS-1$
             }
 
+            return "java.lang.Long"; //$NON-NLS-1$
+
         case Types.BIGINT:
-            return "java.lang.Long";
+
+            if (!f.isUnsigned()) {
+                return "java.lang.Long"; //$NON-NLS-1$
+            }
+
+            return "java.math.BigInteger"; //$NON-NLS-1$
 
         case Types.DECIMAL:
         case Types.NUMERIC:
-            return "java.math.BigDecimal";
+            return "java.math.BigDecimal"; //$NON-NLS-1$
 
         case Types.REAL:
-        case Types.FLOAT:
-            return "java.lang.Float";
+            return "java.lang.Float"; //$NON-NLS-1$
 
+        case Types.FLOAT:
         case Types.DOUBLE:
-            return "java.lang.Double";
+            return "java.lang.Double"; //$NON-NLS-1$
 
         case Types.CHAR:
         case Types.VARCHAR:
         case Types.LONGVARCHAR:
-            return "java.lang.String";
+            return "java.lang.String"; //$NON-NLS-1$
 
         case Types.BINARY:
         case Types.VARBINARY:
         case Types.LONGVARBINARY:
 
             if (!f.isBlob()) {
-                return "java.lang.String";
+                return "java.lang.String"; //$NON-NLS-1$
             } else if (!f.isBinary()) {
-                return "java.lang.String";
+                return "java.lang.String"; //$NON-NLS-1$
             } else {
-                return "java.lang.Object";
+                return "[B"; //$NON-NLS-1$
             }
 
         case Types.DATE:
-            return "java.sql.Date";
+            return "java.sql.Date"; //$NON-NLS-1$
 
         case Types.TIME:
-            return "java.sql.Time";
+            return "java.sql.Time"; //$NON-NLS-1$
 
         case Types.TIMESTAMP:
-            return "java.sql.Timestamp";
+            return "java.sql.Timestamp"; //$NON-NLS-1$
 
         default:
-            return "java.lang.Object";
+            return "java.lang.Object"; //$NON-NLS-1$
         }
     }
 
@@ -202,32 +303,40 @@ public class ResultSetMetaData implements java.sql.ResultSetMetaData {
      * Whats the number of columns in the ResultSet?
      *
      * @return the number
-     * @throws java.sql.SQLException if a database access error occurs
+     *
+     * @throws SQLException if a database access error occurs
      */
-    public int getColumnCount() throws java.sql.SQLException {
-        return fields.length;
+    public int getColumnCount() throws SQLException {
+        return this.fields.length;
     }
 
     /**
      * What is the column's normal maximum width in characters?
      *
      * @param column the first column is 1, the second is 2, etc.
+     *
      * @return the maximum width
-     * @throws java.sql.SQLException if a database access error occurs
+     *
+     * @throws SQLException if a database access error occurs
      */
-    public int getColumnDisplaySize(int column) throws java.sql.SQLException {
-        return getField(column).getLength();
+    public int getColumnDisplaySize(int column) throws SQLException {
+    	Field f = getField(column);
+    	
+        int lengthInBytes = f.getLength();
+        
+        return lengthInBytes / f.getMaxBytesPerCharacter();
     }
 
     /**
-     * What is the suggested column title for use in printouts and
-     * displays?
+     * What is the suggested column title for use in printouts and displays?
      *
      * @param column the first column is 1, the second is 2, etc.
+     *
      * @return the column label
-     * @throws java.sql.SQLException if a database access error occurs
+     *
+     * @throws SQLException if a database access error occurs
      */
-    public String getColumnLabel(int column) throws java.sql.SQLException {
+    public String getColumnLabel(int column) throws SQLException {
         return getColumnName(column);
     }
 
@@ -235,10 +344,12 @@ public class ResultSetMetaData implements java.sql.ResultSetMetaData {
      * What's a column's name?
      *
      * @param column the first column is 1, the second is 2, etc.
+     *
      * @return the column name
-     * @throws java.sql.SQLException if a databvase access error occurs
+     *
+     * @throws SQLException if a databvase access error occurs
      */
-    public String getColumnName(int column) throws java.sql.SQLException {
+    public String getColumnName(int column) throws SQLException {
         return getField(column).getName();
     }
 
@@ -246,11 +357,14 @@ public class ResultSetMetaData implements java.sql.ResultSetMetaData {
      * What is a column's SQL Type? (java.sql.Type int)
      *
      * @param column the first column is 1, the second is 2, etc.
+     *
      * @return the java.sql.Type value
-     * @throws java.sql.SQLException if a database access error occurs
+     *
+     * @throws SQLException if a database access error occurs
+     *
      * @see java.sql.Types
      */
-    public int getColumnType(int column) throws java.sql.SQLException {
+    public int getColumnType(int column) throws SQLException {
         return getField(column).getSQLType();
     }
 
@@ -258,86 +372,92 @@ public class ResultSetMetaData implements java.sql.ResultSetMetaData {
      * Whats is the column's data source specific type name?
      *
      * @param column the first column is 1, the second is 2, etc.
+     *
      * @return the type name
-     * @throws java.sql.SQLException if a database access error occurs
+     *
+     * @throws SQLException if a database access error occurs
      */
     public String getColumnTypeName(int column) throws java.sql.SQLException {
-        int mysqlType = getField(column).getMysqlType();
+    	Field field = getField(column);
+    	
+        int mysqlType = field.getMysqlType();
 
         switch (mysqlType) {
         case MysqlDefs.FIELD_TYPE_DECIMAL:
-            return "DECIMAL";
-
+            return field.isUnsigned() ? "DECIMAL UNSIGNED" : "DECIMAL";
+       
         case MysqlDefs.FIELD_TYPE_TINY:
-            return "TINY";
-
+            return field.isUnsigned() ? "TINYINT UNSIGNED" : "TINYINT";
+       
         case MysqlDefs.FIELD_TYPE_SHORT:
-            return "SHORT";
+            return field.isUnsigned() ? "SMALLINT UNSIGNED" : "SMALLINT";
 
         case MysqlDefs.FIELD_TYPE_LONG:
-            return "LONG";
-
+            return field.isUnsigned() ? "INTEGER UNSIGNED" : "INTEGER";
+        
         case MysqlDefs.FIELD_TYPE_FLOAT:
-            return "FLOAT";
-
+            return field.isUnsigned() ? "FLOAT UNSIGNED" : "FLOAT";
+ 
         case MysqlDefs.FIELD_TYPE_DOUBLE:
-            return "DOUBLE";
+            return field.isUnsigned() ? "DOUBLE UNSIGNED" : "DOUBLE";
 
         case MysqlDefs.FIELD_TYPE_NULL:
-            return "NULL";
-
+            return "NULL"; //$NON-NLS-1$
+    
         case MysqlDefs.FIELD_TYPE_TIMESTAMP:
-            return "TIMESTAMP";
+            return "TIMESTAMP"; //$NON-NLS-1$       
 
         case MysqlDefs.FIELD_TYPE_LONGLONG:
-            return "LONGLONG";
+            return field.isUnsigned() ? "BIGINT UNSIGNED" : "BIGINT";
 
         case MysqlDefs.FIELD_TYPE_INT24:
-            return "INT";
+            return field.isUnsigned() ? "MEDIUMINT UNSIGNED" : "MEDIUMINT";
 
         case MysqlDefs.FIELD_TYPE_DATE:
-            return "DATE";
+            return "DATE"; //$NON-NLS-1$       
 
         case MysqlDefs.FIELD_TYPE_TIME:
-            return "TIME";
+            return "TIME"; //$NON-NLS-1$       
 
         case MysqlDefs.FIELD_TYPE_DATETIME:
-            return "DATETIME";
+            return "DATETIME"; //$NON-NLS-1$      
 
         case MysqlDefs.FIELD_TYPE_TINY_BLOB:
-            return "TINYBLOB";
-
+            return "TINYBLOB"; //$NON-NLS-1$
+        
         case MysqlDefs.FIELD_TYPE_MEDIUM_BLOB:
-            return "MEDIUMBLOB";
-
+            return "MEDIUMBLOB"; //$NON-NLS-1$
+    
         case MysqlDefs.FIELD_TYPE_LONG_BLOB:
-            return "LONGBLOB";
-
+            return "LONGBLOB"; //$NON-NLS-1$
+            
         case MysqlDefs.FIELD_TYPE_BLOB:
-
             if (getField(column).isBinary()) {
-                return "BLOB";
-            } else {
-                return "TEXT";
+                return "BLOB";//$NON-NLS-1$
             }
-
+            
+            return "TEXT";//$NON-NLS-1$
+ 
+        case MysqlDefs.FIELD_TYPE_VARCHAR:
+        	return "VARCHAR"; //$NON-NLS-1$
+        	
         case MysqlDefs.FIELD_TYPE_VAR_STRING:
-            return "VARCHAR";
-
+            return "VARCHAR"; //$NON-NLS-1$
+        
         case MysqlDefs.FIELD_TYPE_STRING:
-            return "CHAR";
+            return "CHAR"; //$NON-NLS-1$
 
         case MysqlDefs.FIELD_TYPE_ENUM:
-            return "ENUM";
+            return "ENUM"; //$NON-NLS-1$
+        
+        case MysqlDefs.FIELD_TYPE_YEAR:
+        	return "YEAR"; //$NON_NLS-1$
 
         case MysqlDefs.FIELD_TYPE_SET:
-            return "SET";
-           
-        case MysqlDefs.FIELD_TYPE_YEAR:
-        	return "YEAR";
-
+            return "SET"; //$NON-NLS-1$
+            
         default:
-            return "UNKNOWN";
+            return "UNKNOWN"; //$NON-NLS-1$
         }
     }
 
@@ -345,10 +465,12 @@ public class ResultSetMetaData implements java.sql.ResultSetMetaData {
      * Is the column a cash value?
      *
      * @param column the first column is 1, the second is 2...
+     *
      * @return true if its a cash column
-     * @throws java.sql.SQLException if a database access error occurs
+     *
+     * @throws SQLException if a database access error occurs
      */
-    public boolean isCurrency(int column) throws java.sql.SQLException {
+    public boolean isCurrency(int column) throws SQLException {
         return false;
     }
 
@@ -356,11 +478,12 @@ public class ResultSetMetaData implements java.sql.ResultSetMetaData {
      * Will a write on this column definately succeed?
      *
      * @param column the first column is 1, the second is 2, etc..
+     *
      * @return true if so
-     * @throws java.sql.SQLException if a database access error occurs
+     *
+     * @throws SQLException if a database access error occurs
      */
-    public boolean isDefinitelyWritable(int column)
-        throws java.sql.SQLException {
+    public boolean isDefinitelyWritable(int column) throws SQLException {
         return isWritable(column);
     }
 
@@ -368,25 +491,29 @@ public class ResultSetMetaData implements java.sql.ResultSetMetaData {
      * Can you put a NULL in this column?
      *
      * @param column the first column is 1, the second is 2...
+     *
      * @return one of the columnNullable values
-     * @throws java.sql.SQLException if a database access error occurs
+     *
+     * @throws SQLException if a database access error occurs
      */
-    public int isNullable(int column) throws java.sql.SQLException {
+    public int isNullable(int column) throws SQLException {
         if (!getField(column).isNotNull()) {
             return java.sql.ResultSetMetaData.columnNullable;
-        } else {
-            return java.sql.ResultSetMetaData.columnNoNulls;
         }
+
+        return java.sql.ResultSetMetaData.columnNoNulls;
     }
 
     /**
      * What is a column's number of decimal digits.
      *
      * @param column the first column is 1, the second is 2...
+     *
      * @return the precision
-     * @throws java.sql.SQLException if a database access error occurs
+     *
+     * @throws SQLException if a database access error occurs
      */
-    public int getPrecision(int column) throws java.sql.SQLException {
+    public int getPrecision(int column) throws SQLException {
         Field f = getField(column);
 
         if (isDecimalType(f.getSQLType())) {
@@ -397,29 +524,46 @@ public class ResultSetMetaData implements java.sql.ResultSetMetaData {
             return f.getLength() + f.getPrecisionAdjustFactor();
         }
 
-        return 0;
+        switch (f.getMysqlType()) {
+        case MysqlDefs.FIELD_TYPE_TINY_BLOB:
+        case MysqlDefs.FIELD_TYPE_BLOB:
+        case MysqlDefs.FIELD_TYPE_MEDIUM_BLOB:
+        case MysqlDefs.FIELD_TYPE_LONG_BLOB:
+        	return f.getLength(); // this may change in the future
+                                  // for now, the server only
+                                  // returns FIELD_TYPE_BLOB for _all_
+                                  // BLOB types, but varying lengths
+                                  // indicating the _maximum_ size
+                                  // for each BLOB type.
+        default:
+        	return f.getLength();
+                
+        }
     }
 
     /**
      * Is the column definitely not writable?
      *
      * @param column the first column is 1, the second is 2, etc.
+     *
      * @return true if so
-     * @throws java.sql.SQLException if a database access error occurs
+     *
+     * @throws SQLException if a database access error occurs
      */
-    public boolean isReadOnly(int column) throws java.sql.SQLException {
-        return false;
+    public boolean isReadOnly(int column) throws SQLException {
+        return getField(column).isReadOnly();
     }
 
     /**
-     * What is a column's number of digits to the right of the
-     * decimal point?
+     * What is a column's number of digits to the right of the decimal point?
      *
      * @param column the first column is 1, the second is 2...
+     *
      * @return the scale
-     * @throws java.sql.SQLException if a database access error occurs
+     *
+     * @throws SQLException if a database access error occurs
      */
-    public int getScale(int column) throws java.sql.SQLException {
+    public int getScale(int column) throws SQLException {
         Field f = getField(column);
 
         if (isDecimalType(f.getSQLType())) {
@@ -430,33 +574,34 @@ public class ResultSetMetaData implements java.sql.ResultSetMetaData {
     }
 
     /**
-     * What is a column's table's schema?  This relies on us knowing
-     * the table name.
-     *
-     * The JDBC specification allows us to return "" if this is not
+     * What is a column's table's schema?  This relies on us knowing the table
+     * name. The JDBC specification allows us to return "" if this is not
      * applicable.
      *
      * @param column the first column is 1, the second is 2...
+     *
      * @return the Schema
-     * @throws java.sql.SQLException if a database access error occurs
+     *
+     * @throws SQLException if a database access error occurs
      */
-    public String getSchemaName(int column) throws java.sql.SQLException {
-        return "";
+    public String getSchemaName(int column) throws SQLException {
+        return ""; //$NON-NLS-1$
     }
 
     /**
-     * Can the column be used in a WHERE clause?  Basically for
-     * this, I split the functions into two types: recognised
-     * types (which are always useable), and OTHER types (which
-     * may or may not be useable).  The OTHER types, for now, I
-     * will assume they are useable.  We should really query the
-     * catalog to see if they are useable.
+     * Can the column be used in a WHERE clause?  Basically for this, I split
+     * the functions into two types: recognised types (which are always
+     * useable), and OTHER types (which may or may not be useable).  The OTHER
+     * types, for now, I will assume they are useable.  We should really query
+     * the catalog to see if they are useable.
      *
      * @param column the first column is 1, the second is 2...
+     *
      * @return true if they can be used in a WHERE clause
-     * @throws java.sql.SQLException if a database access error occurs
+     *
+     * @throws SQLException if a database access error occurs
      */
-    public boolean isSearchable(int column) throws java.sql.SQLException {
+    public boolean isSearchable(int column) throws SQLException {
         return true;
     }
 
@@ -464,10 +609,12 @@ public class ResultSetMetaData implements java.sql.ResultSetMetaData {
      * Is the column a signed number?
      *
      * @param column the first column is 1, the second is 2...
+     *
      * @return true if so
-     * @throws java.sql.SQLException if a database access error occurs
+     *
+     * @throws SQLException if a database access error occurs
      */
-    public boolean isSigned(int column) throws java.sql.SQLException {
+    public boolean isSigned(int column) throws SQLException {
         Field f = getField(column);
         int sqlType = f.getSQLType();
 
@@ -497,10 +644,12 @@ public class ResultSetMetaData implements java.sql.ResultSetMetaData {
      * Whats a column's table's name?
      *
      * @param column the first column is 1, the second is 2...
+     *
      * @return column name, or "" if not applicable
-     * @throws java.sql.SQLException if a database access error occurs
+     *
+     * @throws SQLException if a database access error occurs
      */
-    public String getTableName(int column) throws java.sql.SQLException {
+    public String getTableName(int column) throws SQLException {
         return getField(column).getTableName();
     }
 
@@ -508,13 +657,33 @@ public class ResultSetMetaData implements java.sql.ResultSetMetaData {
      * Is it possible for a write on the column to succeed?
      *
      * @param column the first column is 1, the second is 2, etc.
+     *
      * @return true if so
-     * @throws java.sql.SQLException if a database access error occurs
+     *
+     * @throws SQLException if a database access error occurs
      */
-    public boolean isWritable(int column) throws java.sql.SQLException {
+    public boolean isWritable(int column) throws SQLException {
         return !isReadOnly(column);
     }
 
+    /**
+     * Returns a string representation of this object
+     *
+     * @return ...
+     */
+    public String toString() {
+        StringBuffer toStringBuf = new StringBuffer();
+        toStringBuf.append(super.toString());
+        toStringBuf.append(" - Field level information: "); //$NON-NLS-1$
+
+        for (int i = 0; i < this.fields.length; i++) {
+            toStringBuf.append("\n\t"); //$NON-NLS-1$
+            toStringBuf.append(this.fields[i].toString());
+        }
+
+        return toStringBuf.toString();
+    }
+
     // *********************************************************************
     //
     //                END OF PUBLIC INTERFACE
@@ -525,22 +694,26 @@ public class ResultSetMetaData implements java.sql.ResultSetMetaData {
      * Returns the field instance for the given column index
      *
      * @param columnIndex the column number to retrieve a field instance for
+     *
      * @return the field instance for the given column index
      *
-     * @throws java.sql.SQLException if an error occurs
+     * @throws SQLException if an error occurs
      */
-    protected Field getField(int columnIndex) throws java.sql.SQLException {
-        if ((columnIndex < 1) || (columnIndex > fields.length)) {
-            throw new java.sql.SQLException("Column index out of range.",
-                "S1002");
+    protected Field getField(int columnIndex) throws SQLException {
+        if ((columnIndex < 1) || (columnIndex > this.fields.length)) {
+            throw new SQLException(Messages.getString("ResultSetMetaData.46"), //$NON-NLS-1$
+                SQLError.SQL_STATE_INVALID_COLUMN_NUMBER);
         }
 
-        return fields[columnIndex - 1];
+        return this.fields[columnIndex - 1];
     }
 
     /**
      * Checks if the SQL Type is a Decimal/Number Type
+     *
      * @param type SQL Type
+     *
+     * @return ...
      */
     private static final boolean isDecimalType(int type) {
         switch (type) {
