@@ -1,10 +1,14 @@
 /*
-   Copyright (C) 2002 MySQL AB
+      Copyright (C) 2002-2004 MySQL AB
 
       This program is free software; you can redistribute it and/or modify
-      it under the terms of the GNU General Public License as published by
-      the Free Software Foundation; either version 2 of the License, or
-      (at your option) any later version.
+      it under the terms of version 2 of the GNU General Public License as 
+      published by the Free Software Foundation.
+
+      There are special exceptions to the terms and conditions of the GPL 
+      as it is applied to this software. View the full text of the 
+      exception in file EXCEPTIONS-CONNECTOR-J in the directory of this 
+      software distribution.
 
       This program is distributed in the hope that it will be useful,
       but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,15 +19,19 @@
       along with this program; if not, write to the Free Software
       Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+
+
  */
 package com.mysql.jdbc;
 
+import java.sql.Date;
 import java.sql.Time;
 import java.sql.Timestamp;
 
 import java.util.Calendar;
 import java.util.Collections;
 import java.util.HashMap;
+import java.util.Locale;
 import java.util.Map;
 import java.util.TimeZone;
 
@@ -913,6 +921,8 @@ public class TimeUtil {
      * @param timezoneStr the 'common' timezone name
      *
      * @return the Java timezone name for the given timezone
+     *
+     * @throws IllegalArgumentException DOCUMENT ME!
      */
     public static String getCanoncialTimezone(String timezoneStr) {
         if (timezoneStr == null) {
@@ -922,15 +932,15 @@ public class TimeUtil {
         timezoneStr = timezoneStr.trim();
 
         // Fix windows Daylight/Standard shift JDK doesn't map these (doh)
-        String timezoneStrUC = timezoneStr.toUpperCase();
-        int daylightIndex = timezoneStrUC.indexOf("DAYLIGHT");
+        
+        int daylightIndex = StringUtils.indexOfIgnoreCase(timezoneStr, "DAYLIGHT");
 
         if (daylightIndex != -1) {
             StringBuffer timezoneBuf = new StringBuffer();
             timezoneBuf.append(timezoneStr.substring(0, daylightIndex));
             timezoneBuf.append("Standard");
-            timezoneBuf.append(timezoneStr.substring(daylightIndex
-                    + "DAYLIGHT".length(), timezoneStr.length()));
+            timezoneBuf.append(timezoneStr.substring(daylightIndex +
+                    "DAYLIGHT".length(), timezoneStr.length()));
             timezoneStr = timezoneBuf.toString();
         }
 
@@ -983,29 +993,34 @@ public class TimeUtil {
      * @return the timestamp changed to the timezone 'toTz'
      */
     public static Timestamp changeTimezone(Connection conn, Timestamp tstamp,
-        TimeZone fromTz, TimeZone toTz) {
-        if ((conn != null) && conn.useTimezone()) {
+        TimeZone fromTz, TimeZone toTz, boolean rollForward) {
+        if ((conn != null) && conn.getUseTimezone()) {
             // Convert the timestamp from GMT to the server's timezone
             Calendar fromCal = Calendar.getInstance(fromTz);
             fromCal.setTime(tstamp);
 
-            int fromOffset = fromCal.get(Calendar.ZONE_OFFSET)
-                + fromCal.get(Calendar.DST_OFFSET);
+            int fromOffset = fromCal.get(Calendar.ZONE_OFFSET) +
+                fromCal.get(Calendar.DST_OFFSET);
             Calendar toCal = Calendar.getInstance(toTz);
             toCal.setTime(tstamp);
 
-            int toOffset = toCal.get(Calendar.ZONE_OFFSET)
-                + toCal.get(Calendar.DST_OFFSET);
-            int offsetDiff = toOffset - fromOffset;
+            int toOffset = toCal.get(Calendar.ZONE_OFFSET) +
+                toCal.get(Calendar.DST_OFFSET);
+            int offsetDiff = fromOffset - toOffset;
             long toTime = toCal.getTime().getTime();
-            toTime += offsetDiff;
+            
+            if (rollForward || (conn.isServerTzUTC() && !conn.isClientTzUTC())) {
+            	toTime += offsetDiff;
+            } else {
+            	toTime -= offsetDiff;
+            }
 
             Timestamp changedTimestamp = new Timestamp(toTime);
 
             return changedTimestamp;
-        } else {
-            return tstamp;
         }
+
+        return tstamp;
     }
 
     /**
@@ -1019,28 +1034,100 @@ public class TimeUtil {
      * @return the times changed to the timezone 'toTz'
      */
     public static Time changeTimezone(Connection conn, Time t, TimeZone fromTz,
-        TimeZone toTz) {
-        if ((conn != null) && conn.useTimezone()) {
+        TimeZone toTz, boolean rollForward) {
+        if ((conn != null) && conn.getUseTimezone()) {
             // Convert the timestamp from GMT to the server's timezone
             Calendar fromCal = Calendar.getInstance(fromTz);
             fromCal.setTime(t);
 
-            int fromOffset = fromCal.get(Calendar.ZONE_OFFSET)
-                + fromCal.get(Calendar.DST_OFFSET);
+            int fromOffset = fromCal.get(Calendar.ZONE_OFFSET) +
+                fromCal.get(Calendar.DST_OFFSET);
             Calendar toCal = Calendar.getInstance(toTz);
             toCal.setTime(t);
 
-            int toOffset = toCal.get(Calendar.ZONE_OFFSET)
-                + toCal.get(Calendar.DST_OFFSET);
-            int offsetDiff = toOffset - fromOffset;
+            int toOffset = toCal.get(Calendar.ZONE_OFFSET) +
+                toCal.get(Calendar.DST_OFFSET);
+            int offsetDiff = fromOffset - toOffset;
             long toTime = toCal.getTime().getTime();
-            toTime += offsetDiff;
+            
+            if (rollForward || (conn.isServerTzUTC() && !conn.isClientTzUTC())) {
+            	toTime += offsetDiff;
+            } else {
+            	toTime -= offsetDiff;
+            }
 
             Time changedTime = new Time(toTime);
 
             return changedTime;
-        } else {
-            return t;
         }
+
+        return t;
+    }
+
+    //
+    // WARN! You must externally synchronize these calendar instances
+    // See ResultSet.fastDateCreate() for an example
+    //
+    final static Date fastDateCreate(Calendar cal, int year, int month, int day) {
+        cal.clear();
+
+        // why-oh-why is this different than java.util.date,
+        // in the year part, but it still keeps the silly '0'
+        // for the start month????
+        cal.set(year, month - 1, day, 0, 0, 0);
+
+        long dateAsMillis = 0;
+
+        try {
+            dateAsMillis = cal.getTimeInMillis();
+        } catch (IllegalAccessError iae) {
+            // Must be on JDK-1.3.1 or older....
+            dateAsMillis = cal.getTime().getTime();
+        }
+
+        return new Date(dateAsMillis);
+    }
+
+    final static Time fastTimeCreate(Calendar cal, int hour, int minute,
+        int second) {
+        cal.clear();
+
+        // Set 'date' to epoch of Jan 1, 1970
+        cal.set(1970, 0, 1, hour, minute, second);
+
+        long timeAsMillis = 0;
+
+        try {
+            timeAsMillis = cal.getTimeInMillis();
+        } catch (IllegalAccessError iae) {
+            // Must be on JDK-1.3.1 or older....
+            timeAsMillis = cal.getTime().getTime();
+        }
+
+        return new Time(timeAsMillis);
+    }
+
+    final static Timestamp fastTimestampCreate(Calendar cal, int year,
+        int month, int day, int hour, int minute, int seconds, int secondsPart) {
+        cal.clear();
+
+        //			why-oh-why is this different than java.util.date,
+        // in the year part, but it still keeps the silly '0'
+        // for the start month????
+        cal.set(year, month - 1, day, hour, minute, seconds);
+
+        long tsAsMillis = 0;
+
+        try {
+            tsAsMillis = cal.getTimeInMillis();
+        } catch (IllegalAccessError iae) {
+            // Must be on JDK-1.3.1 or older....
+            tsAsMillis = cal.getTime().getTime();
+        }
+
+        Timestamp ts = new Timestamp(tsAsMillis);
+        ts.setNanos(secondsPart);
+
+        return ts;
     }
 }
